@@ -48,12 +48,14 @@ class MySQLConnection(DBAPI):
         # MySQLdb > 1.2.1: both ascii and unicode
         self.need_unicode = (self.module.version_info[:3] >= (1, 2, 1)) and (self.module.version_info[:3] < (1, 2, 2))
 
+        self._server_version = None
+        self._can_use_microseconds = None
         DBAPI.__init__(self, **kw)
 
+    @classmethod
     def _connectionFromParams(cls, user, password, host, port, path, args):
         return cls(db=path.strip('/'), user=user or '', password=password or '',
                    host=host or 'localhost', port=port or 0, **args)
-    _connectionFromParams = classmethod(_connectionFromParams)
 
     def makeConnection(self):
         dbEncoding = self.dbEncoding
@@ -162,19 +164,19 @@ class MySQLConnection(DBAPI):
             self.printDebug(conn, id, 'QueryIns', 'result')
         return id
 
+    @classmethod
     def _queryAddLimitOffset(cls, query, start, end):
         if not start:
             return "%s LIMIT %i" % (query, end)
         if not end:
             return "%s LIMIT %i, -1" % (query, start)
         return "%s LIMIT %i, %i" % (query, start, end-start)
-    _queryAddLimitOffset = classmethod(_queryAddLimitOffset)
 
     def createReferenceConstraint(self, soClass, col):
         return col.mysqlCreateReferenceConstraint()
 
     def createColumn(self, soClass, col):
-        return col.mysqlCreateSQL()
+        return col.mysqlCreateSQL(self)
 
     def createIndexSQL(self, soClass, index):
         return index.mysqlCreateIndexSQL(soClass)
@@ -202,7 +204,7 @@ class MySQLConnection(DBAPI):
     def addColumn(self, tableName, column):
         self.query('ALTER TABLE %s ADD COLUMN %s' %
                    (tableName,
-                    column.mysqlCreateSQL()))
+                    column.mysqlCreateSQL(self)))
 
     def delColumn(self, sqlmeta, column):
         self.query('ALTER TABLE %s DROP COLUMN %s' % (sqlmeta.table, column.dbName))
@@ -303,3 +305,35 @@ class MySQLConnection(DBAPI):
 
     def dropDatabase(self):
         self._createOrDropDatabase(op="DROP")
+
+    def server_version(self):
+        if self._server_version is not None:
+            return self._server_version
+        try:
+            server_version = self.queryOne("SELECT VERSION()")[0]
+            server_version = server_version.split('-', 1)
+            db_tag = "MySQL"
+            if len(server_version) == 2:
+                if "MariaDB" in server_version[1]:
+                    db_tag = "MariaDB"
+                server_version = server_version[0]
+            server_version = tuple(int(v) for v in server_version.split('.'))
+            server_version = (server_version, db_tag)
+        except:
+            server_version = None # unknown
+        self._server_version = server_version
+        return server_version
+
+    def can_use_microseconds(self):
+        if self._can_use_microseconds is not None:
+            return self._can_use_microseconds
+        server_version = self.server_version()
+        if server_version is None:
+            return None
+        server_version, db_tag = server_version
+        if db_tag == "MariaDB":
+            can_use_microseconds = (server_version >= (5, 3, 0))
+        else: # MySQL
+            can_use_microseconds = (server_version >= (5, 6, 4))
+        self._can_use_microseconds = can_use_microseconds
+        return can_use_microseconds
