@@ -6,9 +6,13 @@ from sqlobject.converters import registerConverter
 from sqlobject.dberrors import *
 
 class ErrorMessage(str):
-    def __new__(cls, e):
-        obj = str.__new__(cls, e[0])
-        obj.code = None
+    def __new__(cls, e, append_msg=''):
+        obj = str.__new__(cls, e[0] + append_msg)
+        if e.__module__ == 'psycopg2':
+            obj.code = getattr(e, 'pgcode', None)
+            obj.error = getattr(e, 'pgerror', None)
+        else:
+            obj.code = obj.error = None
         obj.module = e.__module__
         obj.exception = e.__class__.__name__
         return obj
@@ -139,7 +143,7 @@ class PostgresConnection(DBAPI):
             else:
                 conn = self.module.connect(**self.dsn_dict)
         except self.module.OperationalError, e:
-            raise OperationalError("%s; used connection string %r" % (e, self.dsn))
+            raise OperationalError(ErrorMessage(e, "used connection string %r" % self.dsn))
 
         # For printDebug in _executeRetry
         self._connectionNumbers[id(conn)] = self._connectionCount
@@ -189,15 +193,20 @@ class PostgresConnection(DBAPI):
         sequenceName = soInstance.sqlmeta.idSequence or \
                                '%s_%s_seq' % (table, idName)
         c = conn.cursor()
+        if id is not None:
+            names = [idName] + names
+            values = [id] + values
+        if names and values:
+            q = self._insertSQL(table, names, values)
+        else:
+            q = "INSERT INTO %s DEFAULT VALUES" % table
         if id is None:
-            self._executeRetry(conn, c, "SELECT NEXTVAL('%s')" % sequenceName)
-            id = c.fetchone()[0]
-        names = [idName] + names
-        values = [id] + values
-        q = self._insertSQL(table, names, values)
+            q += " RETURNING " + idName
         if self.debug:
             self.printDebug(conn, q, 'QueryIns')
         self._executeRetry(conn, c, q)
+        if id is None:
+            id = c.fetchone()[0]
         if self.debugOutput:
             self.printDebug(conn, id, 'QueryIns', 'result')
         return id
@@ -271,6 +280,17 @@ class PostgresConnection(DBAPI):
             AND pg_index.indisprimary
         """
 
+        otherKeyQuery = """
+        SELECT pg_index.indisprimary,
+            pg_catalog.pg_get_indexdef(pg_index.indexrelid)
+        FROM pg_catalog.pg_class c, pg_catalog.pg_class c2,
+            pg_catalog.pg_index AS pg_index
+        WHERE c.relname = %s
+            AND c.oid = pg_index.indrelid
+            AND pg_index.indexrelid = c2.oid
+            AND NOT pg_index.indisprimary
+        """
+
         keyData = self.queryAll(keyQuery % self.sqlrepr(tableName))
         keyRE = re.compile(r"\((.+)\) REFERENCES (.+)\(")
         keymap = {}
@@ -289,11 +309,26 @@ class PostgresConnection(DBAPI):
             assert match, "Unparseable contraint definition: %r" % indexDef
             assert primaryKey is None, "Already found primary key (%r), then found: %r" % (primaryKey, indexDef)
             primaryKey = match.group(1)
+        if primaryKey is None:
+            # VIEWs don't have PRIMARY KEYs - accept help from user
+            primaryKey = soClass.sqlmeta.idName
         assert primaryKey, "No primary key found in table %r" % tableName
         if primaryKey.startswith('"'):
             assert primaryKey.endswith('"')
             primaryKey = primaryKey[1:-1]
 
+        otherData = self.queryAll(otherKeyQuery % self.sqlrepr(tableName))
+        otherRE = primaryRE
+        otherKeys = []
+        for isPrimary, indexDef in otherData:
+            match = otherRE.search(indexDef)
+            assert match, "Unparseable constraint definition: %r" % indexDef
+            otherKey = match.group(1)
+            if otherKey.startswith('"'):
+                assert otherKey.endswith('"')
+                otherKey = otherKey[1:-1]
+            otherKeys.append(otherKey)
+
         colData = self.queryAll(colQuery % self.sqlrepr(tableName))
         results = []
         if self.unicodeCols:
@@ -320,6 +355,8 @@ class PostgresConnection(DBAPI):
                 kw['default'] = self.defaultFromSchema(colClass, defaultstr)
             elif not notnull:
                 kw['default'] = None
+            if field in otherKeys:
+                kw['alternateID'] = True
             results.append(colClass(**kw))
         return results
 
